#include<ilcplex/cplex.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#define INPUT "input"
void error(char *s)
{
	fprintf(stderr, "%s\n", s);
	printf("Bilo koji karakter za izlaz iz programa...\n");
	getchar();
	exit(EXIT_FAILURE);
}

int main()
{
	
		char name[80];
		sprintf(name, "%s.txt", INPUT);
		FILE *fin = fopen(name, "r");
		if (fin == NULL)
			error("Error opening input");

		int p, q; // broj objekata koje ce firma A uspostaviti, broj objekata koji vec postoje (firme B)
		int I, J; // broj potrosaca, broj potencijalnih lokacija
		double *w; // potraznje potrosaca
		int* cntP;
		int *cntT;
		double **P;
		double **T;
		/* ---FORMAT---
		I J p
		for each i
		wi
		for each i
		cntPi Pi
		cntTi Ti
		*/
		
		//--- UCITAVANJE ---//
		fscanf(fin, "%d %d %d %d", &I, &J, &p, &q);
		w = (double*)malloc(I * sizeof(double));
		for (int i = 0; i < I; i++)
		{
			fscanf(fin, "%lf", &w[i]);
		}

		cntP = malloc(I * sizeof(int));
		cntT = malloc(I * sizeof(int));
		P = malloc(I * sizeof(double*));
		T = malloc(I * sizeof(double*));
		
		for (int i = 0; i < I; i++)
		{
			fscanf(fin, "%d", &cntP[i]);
			P[i] = malloc(cntP[i] * sizeof(double));
			for (int j = 0; j < cntP[i]; j++)
				fscanf(fin, "%lf", &P[i][j]);
			fscanf(fin, "%d", &cntT[i]);
			T[i] = malloc(cntT[i] * sizeof(double));
			for (int j = 0; j < cntT[i]; j++)
				fscanf(fin, "%lf", &T[i][j]);
		}

		if (fclose(fin) != 0)
			error("Error closing input");
		/*
		printf("%d %d %d\n", I, J, p);
		printf("\nP\n_________________________\n");
		for (int i = 0; i < 1; i++)
		{
		for (int j = 0; j < cntP[i]; j++)
		printf("%lf ", P[i][j]);
		printf("\n");
		}
		printf("\nT\n_________________________\n");
		for (int i = 0; i < 1; i++)
		{
		for (int j = 0; j < cntT[i]; j++)
		printf("%lf ", T[i][j]);
		printf("\n");
		}
		*/

		//--- UCITAVANJE ---//

		//--- CPLEX ---//
		int status;
		CPXENVptr env = CPXopenCPLEX(&status);
		if (status != 0)
			error("CPXopenCPLEX error");
		CPXLPptr lp = CPXcreateprob(env, &status, "MAXCAP");
		if (status != 0)
			error("CPXcreateprob error");
		CPXchgobjsen(env, lp, CPX_MAX);
		if (status != 0)
			error("CPXchgobjsen error");

		int ccnt = J + 2 * I;
		double *coefs = (double *)malloc(ccnt * sizeof(double)); //fja cilja
		double *ub = (double *)malloc(ccnt * sizeof(double));
		char *type = (char *)malloc(ccnt * sizeof(char));

		for (int j = 0; j < J; j++)
		{
			coefs[j] = 0;
			type[j] = 'B';
			ub[j] = 1;
		}
		for (int i = 0; i < I; i++)
		{
			coefs[J + i] = w[i];
			coefs[J + I + i] = w[i] / 2;
			type[J + i] = type[J + I + i] = 'B';
			ub[J + i] = ub[J + I + i] = 1;
		}

		CPXnewcols(env, lp, ccnt, coefs, NULL, ub, type, NULL);
		if (status != 0)
			error("CPXnewcols error");
		free(coefs);
		free(type);
		free(ub);

		//--- USLOVI ---//
		//--- USLOV 1 ---//
		// yi <= sum(xj) //
		int *rmatind;
		double *rmatval;
		double *rhs = (double *)malloc(sizeof(double));
		int *rmatbeg = (int *)malloc(sizeof(int));
		char *sense = (char *)malloc(sizeof(char));

		for (int i = 0; i < I; i++)
		{
			rmatind = (int *)malloc((cntP[i] + 1) * sizeof(int));
			rmatval = (double *)malloc((cntP[i] + 1) * sizeof(double));
			rhs[0] = 0;
			sense[0] = 'L';
			rmatbeg[0] = 0;
			for (int j = 0; j < cntP[i]; j++)
			{
				rmatval[j] = -1;
				rmatind[j] = P[i][j];
			}
			rmatval[cntP[i]] = 1;
			rmatind[cntP[i]] = J + i;
			status = CPXaddrows(env, lp, 0, 1, cntP[i] + 1, rhs, sense, rmatbeg, rmatind, rmatval, NULL, NULL);
			if (status != 0)
			{
				char buffer[256];
				CPXgeterrorstring(env, status, buffer);
				fprintf(stderr, "CPXaddrows1 error\n");
				error(buffer);
			}
			free(rmatind);
			free(rmatval);
		}
		free(rhs);
		free(sense);
		free(rmatbeg);
		//--- USLOV 1 ---//

		//--- USLOV 2 ---//
		//zi <= sum xj, j in Ti  
		rhs = (double *)malloc(sizeof(double));
		rmatbeg = (int *)malloc(sizeof(int));
		sense = (char *)malloc(sizeof(char));

		for (int i = 0; i < I; i++)
		{
			rmatind = (int *)malloc((cntT[i] + 1) * sizeof(int));
			rmatval = (double *)malloc((cntT[i] + 1) * sizeof(double));
			rhs[0] = 0;
			sense[0] = 'L';
			rmatbeg[0] = 0;
			for (int j = 0; j < cntT[i]; j++)
			{
				rmatval[j] = -1;
				rmatind[j] = T[i][j];
			}
			rmatval[cntT[i]] = 1;
			rmatind[cntT[i]] = J + I + i;
			status = CPXaddrows(env, lp, 0, 1, cntT[i] + 1, rhs, sense, rmatbeg, rmatind, rmatval, NULL, NULL);
			if (status != 0)
			{
				char buffer[256];
				CPXgeterrorstring(env, status, buffer);
				fprintf(stderr, "CPXaddrows2 error");
				error(buffer);
			}
			free(rmatind);
			free(rmatval);
		}
		free(rhs);
		free(sense);
		free(rmatbeg);
		//--- USLOV 2 ---//

		//--- USLOV 3 ---//
		//yi + zi <= 1  
		rmatind = (int *)malloc(2 * sizeof(int));
		rmatval = (double *)malloc(2 * sizeof(double));
		rhs = (double *)malloc(sizeof(double));
		rmatbeg = (int *)malloc(sizeof(int));
		sense = (char *)malloc(sizeof(char));

		for (int i = 0; i < I; i++)
		{
			rhs[0] = 1;
			sense[0] = 'L';
			rmatbeg[0] = 0;
			rmatval[0] = 1;
			rmatval[1] = 1;
			rmatind[0] = J + i;
			rmatind[1] = J + I + i;
			CPXaddrows(env, lp, 0, 1, 2, rhs, sense, rmatbeg, rmatind, rmatval, NULL, NULL);
			if (status != 0)
			{
				char buffer[256];
				CPXgeterrorstring(env, status, buffer);
				fprintf(stderr, "CPXaddrows3 error");
				error(buffer);
			}
		}
		free(rhs);
		free(sense);
		free(rmatind);
		free(rmatbeg);
		free(rmatval);
		//--- USLOV 3 ---//

		//--- USLOV 4 ---//
		//sum(xj)= p  
		rmatind = (int *)malloc(J * sizeof(int));
		rmatval = (double *)malloc(J * sizeof(double));
		rhs = (double *)malloc(sizeof(double));
		rmatbeg = (int *)malloc(sizeof(int));
		sense = (char *)malloc(sizeof(char));
		rhs[0] = p;
		sense[0] = 'E';
		rmatbeg[0] = 0;
		for (int j = 0; j < J; j++)
		{
			rmatind[j] = j;
			rmatval[j] = 1;
		}
		CPXaddrows(env, lp, 0, 1, J, rhs, sense, rmatbeg, rmatind, rmatval, NULL, NULL);
		if (status != 0)
		{
			char buffer[256];
			CPXgeterrorstring(env, status, buffer);
			fprintf(stderr, "CPXaddrows4 error");
			error(buffer);
		}
		free(rhs);
		free(sense);
		free(rmatind);
		free(rmatbeg);
		free(rmatval);
		//--- USLOV 4 ---//
		//--- USLOVI ---//

		//--- RESENJE ---//
		FILE *sol = fopen("out.txt", "a");
		if (sol == NULL)
			error("error opening output file");

		time_t startTime = clock();
		status = CPXchgprobtype(env, lp, CPXPROB_MILP);
		if (status != 0)
		{
			char buffer[80];
			CPXgeterrorstring(env, status, buffer);
			fprintf(stderr, "CPXchgprobtype error");
			error(buffer);
		}
		status = CPXsetdblparam(env, CPX_PARAM_TILIM, 14400.0);
		if (status != 0)
		{
			char buffer[80];
			CPXgeterrorstring(env, status, buffer);
			fprintf(stderr, "CPXsetdblparam error");
			error(buffer);
		}
		status = CPXmipopt(env, lp);
		if (status != 0)
		{
			char buffer[80];
			CPXgeterrorstring(env, status, buffer);
			fprintf(stderr, "CPXmipopt error");
			error(buffer);
		}

		fprintf(sol, "%s\n\t I: %d J: %d p: %d q: %d\n", name, I, J, p, q);
		int lpstat = CPXgetstat(env, lp);
		if (lpstat == 0)
			fprintf(sol, "nema resenje\n");
		else
		{
			double objval;
			status = CPXgetobjval(env, lp, &objval);
			if (status != 0)
				fprintf(sol, "GRESKA - -");
			else
			{
				fprintf(sol, "\topt: %g\t", objval);
				clock_t endTime = clock();
				fprintf(sol, "Vreme(sec): %lf \t", ((double)endTime - startTime) / CLOCKS_PER_SEC);
				fprintf(sol, "Iteracija: %d\t", CPXgetmipitcnt(env, lp));
				fprintf(sol, "Cvorova: %d\n\n", CPXgetnodecnt(env, lp));
			}
		}
		CPXfreeprob(env, &lp);
		CPXcloseCPLEX(&env);
		//--- CPLEX ---//

		for (int i = 0; i < I; i++)
		{
			free(P[i]);
			free(T[i]);
		}
		free(P);
		free(T);
		free(cntP);
		free(cntT);
		free(w);

	//printf("Bilo koji karakter za kraj programa...\n");
	//getchar();
	return 0;
}